@@ -39,6 +39,7 @@
 #include "teddy_engine_description.h"
 #include "grey.h"
 #include "ue2common.h"
+#include "hwlm/hwlm_build.h"
 #include "util/alloc.h"
 #include "util/compare.h"
 #include "util/dump_mask.h"
@@ -495,14 +496,34 @@ FDRCompiler::build(pair<aligned_unique_ptr<u8>, size_t> &link) {
 
 } // namespace
 
+static
+size_t maxMaskLen(const vector<hwlmLiteral> &lits) {
+    size_t rv = 0;
+    for (const auto &lit : lits) {
+        rv = max(rv, lit.msk.size());
+    }
+    return rv;
+}
+
+static
+void setHistoryRequired(hwlmStreamingControl &stream_ctl,
+                        const vector<hwlmLiteral> &lits) {
+    size_t max_mask_len = maxMaskLen(lits);
+
+    // we want enough history to manage the longest literal and the longest
+    // mask.
+    stream_ctl.literal_history_required = max(maxLen(lits), max_mask_len) - 1;
+}
+
 static
 aligned_unique_ptr<FDR>
 fdrBuildTableInternal(const vector<hwlmLiteral> &lits, bool make_small,
                       const target_t &target, const Grey &grey, u32 hint,
                       hwlmStreamingControl *stream_control) {
     pair<aligned_unique_ptr<u8>, size_t> link(nullptr, 0);
+
     if (stream_control) {
-        link = fdrBuildTableStreaming(lits, *stream_control);
+        setHistoryRequired(*stream_control, lits);
     }
 
     DEBUG_PRINTF("cpu has %s\n", target.has_avx2() ? "avx2" : "no-avx2");
