@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015-2016, Intel Corporation
+ * Copyright (c) 2015-2017, Intel Corporation
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -68,8 +68,6 @@
 #include <utility>
 #include <vector>
 #include <boost/core/noncopyable.hpp>
-#include <boost/graph/reverse_graph.hpp>
-#include <boost/graph/topological_sort.hpp>
 
 #define NDEBUG_PRINTF(x, ...) \
     do { if (0) { DEBUG_PRINTF(x,  ## __VA_ARGS__); } } while (0)
@@ -540,7 +538,7 @@ void getRegionRoseLiterals(const NGHolder &g,
         DEBUG_PRINTF("inspecting region %u\n", region);
         set<ue2_literal> s;
         for (auto v : vv) {
-            DEBUG_PRINTF("   exit vertex: %u\n", g[v].index);
+            DEBUG_PRINTF("   exit vertex: %zu\n", g[v].index);
             /* Note: RHS can not be depended on to take all subsequent revisits
              * to this vertex */
             set<ue2_literal> ss = getLiteralSet(g, v, false);
@@ -575,8 +573,7 @@ void gatherBackEdges(const NGHolder &g,
                      ue2::unordered_map<NFAVertex, vector<NFAVertex>> *out) {
     set<NFAEdge> backEdges;
     BackEdges<set<NFAEdge>> be(backEdges);
-    depth_first_search(g.g, visitor(be).root_vertex(g.start).vertex_index_map(
-                                get(&NFAGraphVertexProps::index, g.g)));
+    depth_first_search(g, visitor(be).root_vertex(g.start));
 
     for (const auto &e : backEdges) {
         (*out)[source(e, g)].push_back(target(e, g));
@@ -759,7 +756,7 @@ unique_ptr<VertLitInfo> LitCollection::pickNext() {
         unique_ptr<VertLitInfo> rv = move(lits.back());
         lits.pop_back();
         poisonCandidates(*rv);
-        DEBUG_PRINTF("best is '%s' %u a%d t%d\n",
+        DEBUG_PRINTF("best is '%s' %zu a%d t%d\n",
                      dumpString(*(rv->lit.begin())).c_str(),
                      g[rv->vv.front()].index,
                      (int)createsAnchoredLHS(g, rv->vv, depths, grey),
@@ -811,6 +808,7 @@ bool can_match(const NGHolder &g, const ue2_literal &lit, bool overhang_ok) {
 
 u32 removeTrailingLiteralStates(NGHolder &g, const ue2_literal &lit,
                                 u32 max_delay, bool overhang_ok) {
+    assert(isCorrectlyTopped(g));
     if (max_delay == MO_INVALID_IDX) {
         max_delay--;
     }
@@ -864,8 +862,6 @@ u32 removeTrailingLiteralStates(NGHolder &g, const ue2_literal &lit,
     assert(delay <= lit.length());
     DEBUG_PRINTF("managed delay %u (of max %u)\n", delay, max_delay);
 
-    // For determinism, we make sure that we create these edges from vertices
-    // in index-sorted order.
     set<NFAVertex> pred;
     for (auto v : curr) {
         insert(&pred, inv_adjacent_vertices_range(v, g));
@@ -874,16 +870,17 @@ u32 removeTrailingLiteralStates(NGHolder &g, const ue2_literal &lit,
     clear_in_edges(g.accept, g);
     clearReports(g);
 
-    vector<NFAVertex> verts(pred.begin(), pred.end());
-    sort(verts.begin(), verts.end(), VertexIndexOrdering<NGHolder>(g));
-
-    for (auto v : verts) {
-        add_edge(v, g.accept, g);
+    for (auto v : pred) {
+        NFAEdge e = add_edge(v, g.accept, g);
         g[v].reports.insert(0);
+        if (is_triggered(g) && v == g.start) {
+            g[e].tops.insert(DEFAULT_TOP);
+        }
     }
 
     pruneUseless(g);
     assert(allMatchStatesHaveReports(g));
+    assert(isCorrectlyTopped(g));
 
     DEBUG_PRINTF("graph has %zu vertices left\n", num_vertices(g));
     return delay;
@@ -892,6 +889,7 @@ u32 removeTrailingLiteralStates(NGHolder &g, const ue2_literal &lit,
 void restoreTrailingLiteralStates(NGHolder &g, const ue2_literal &lit,
                                   u32 delay, const vector<NFAVertex> &preds) {
     assert(delay <= lit.length());
+    assert(isCorrectlyTopped(g));
     DEBUG_PRINTF("adding on '%s' %u\n", dumpString(lit).c_str(), delay);
 
     NFAVertex prev = g.accept;
@@ -906,7 +904,10 @@ void restoreTrailingLiteralStates(NGHolder &g, const ue2_literal &lit,
     }
 
     for (auto v : preds) {
-        add_edge(v, prev, g);
+        NFAEdge e = add_edge(v, prev, g);
+        if (v == g.start && is_triggered(g)) {
+            g[e].tops.insert(DEFAULT_TOP);
+        }
     }
 
     // Every predecessor of accept must have a report.
@@ -914,9 +915,10 @@ void restoreTrailingLiteralStates(NGHolder &g, const ue2_literal &lit,
         g[u].reports.insert(0);
     }
 
-    g.renumberVertices();
-    g.renumberEdges();
+    renumber_vertices(g);
+    renumber_edges(g);
     assert(allMatchStatesHaveReports(g));
+    assert(isCorrectlyTopped(g));
 }
 
 void restoreTrailingLiteralStates(NGHolder &g, const ue2_literal &lit,
@@ -1144,7 +1146,7 @@ void deanchorIfNeeded(NGHolder &g, bool *orig_anch) {
     succ_g.erase(g.startDs);
 
     for (auto v : adjacent_vertices_range(g.start, g)) {
-        DEBUG_PRINTF("inspecting cand %u || =%zu\n", g[v].index,
+        DEBUG_PRINTF("inspecting cand %zu || =%zu\n", g[v].index,
                      g[v].char_reach.size());
 
         if (v == g.startDs || !g[v].char_reach.all()) {
@@ -1162,7 +1164,7 @@ void deanchorIfNeeded(NGHolder &g, bool *orig_anch) {
             }
             clear_vertex(v, g);
             remove_vertex(v, g);
-            g.renumberVertices();
+            renumber_vertices(g);
             return;
         }
 
@@ -1693,7 +1695,7 @@ void splitEdgesByCut(RoseInGraph &ig, const vector<RoseInEdge> &to_cut,
             /* TODO need to update v_mapping (if we were doing more cuts) */
         }
 
-        DEBUG_PRINTF("splitting on pivot %u\n", h[pivot].index);
+        DEBUG_PRINTF("splitting on pivot %zu\n", h[pivot].index);
         ue2::unordered_map<NFAVertex, NFAVertex> temp_map;
         shared_ptr<NGHolder> new_lhs = make_shared<NGHolder>();
         splitLHS(h, pivot, new_lhs.get(), &temp_map);
@@ -1766,8 +1768,8 @@ bool doNetflowCut(RoseInGraph &ig, const vector<RoseInEdge> &to_cut,
         return false;
     }
 
-    h.renumberVertices();
-    h.renumberEdges();
+    renumber_vertices(h);
+    renumber_edges(h);
     /* Step 1: Get scores for all edges */
     vector<u64a> scores = scoreEdges(h); /* scores by edge_index */
     /* Step 2: poison scores for edges covered by successor literal */
@@ -2366,12 +2368,17 @@ void makeNocaseWithPrefixMask(RoseInGraph &g, RoseInVertex v) {
 
                 h[ds].char_reach = CharReach::dot();
 
-                add_edge(h.start, ds, h);
+                NFAEdge e_start_to_ds = add_edge(h.start, ds, h);
                 add_edge(ds, ds, h);
                 add_edge(ds, h.accept, h);
                 h[h.start].reports.insert(0);
                 h[ds].reports.insert(0);
+
+                if (g[u].type == RIV_LITERAL) {
+                    h[e_start_to_ds].tops.insert(DEFAULT_TOP);
+                }
             } else {
+                assert(g[u].type == RIV_ANCHORED_START);
                 add_edge(h.start, h.accept, h);
                 h[h.start].reports.insert(0);
             }
@@ -2406,14 +2413,14 @@ void explodeLiteral(RoseInGraph &g, RoseInVertex v,
         g[v_new].s = lit;
 
         for (const auto &e : in_edges_range(v, g)) {
-            RoseInEdge e2 = add_edge(source(e, g), v_new, g[e], g).first;
+            RoseInEdge e2 = add_edge(source(e, g), v_new, g[e], g);
             // FIXME: are we safe to share graphs here? For now, make our very
             // own copy.
             g[e2].graph = makeGraphCopy(g[e].graph.get());
         }
 
         for (const auto &e : out_edges_range(v, g)) {
-            RoseInEdge e2 = add_edge(v_new, target(e, g), g[e], g).first;
+            RoseInEdge e2 = add_edge(v_new, target(e, g), g[e], g);
             // FIXME: are we safe to share graphs here? For now, make our very
             // own copy.
             g[e2].graph = makeGraphCopy(g[e].graph.get());
@@ -2565,7 +2572,7 @@ bool followedByStar(const vector<NFAVertex> &vv, const NGHolder &g) {
 
 static
 bool isEodPrefixCandidate(const NGHolder &g) {
-    if (hasGreaterInDegree(0, g.accept, g)) {
+    if (in_degree(g.accept, g)) {
         DEBUG_PRINTF("graph isn't eod anchored\n");
         return false;
     }
@@ -2636,7 +2643,7 @@ void processEodPrefixes(RoseInGraph &g) {
         }
 
         // TODO: handle cases with multiple out-edges.
-        if (hasGreaterOutDegree(1, source(e, g), g)) {
+        if (out_degree(source(e, g), g) > 1) {
             continue;
         }
 
@@ -2663,7 +2670,7 @@ void processEodPrefixes(RoseInGraph &g) {
     }
 
     for (auto v : accepts) {
-        if (!hasGreaterInDegree(0, v, g)) {
+        if (!in_degree(v, g)) {
             remove_vertex(v, g);
         }
     }
@@ -2805,6 +2812,7 @@ unique_ptr<RoseInGraph> buildRose(const NGHolder &h, bool desperation,
 
     dumpPreRoseGraph(ig, cc.grey);
 
+    renumber_vertices(ig);
     calcVertexOffsets(ig);
     return igp;
 }
@@ -2821,6 +2829,7 @@ void desperationImprove(RoseInGraph &ig, const CompileContext &cc) {
     handleLongMixedSensitivityLiterals(ig);
     dedupe(ig);
     pruneUseless(ig);
+    renumber_vertices(ig);
     calcVertexOffsets(ig);
 }
 
@@ -2831,8 +2840,7 @@ bool splitOffRose(RoseBuild &rose, const NGHolder &h, bool prefilter,
     }
 
     // We should have at least one edge into accept or acceptEod!
-    assert(hasGreaterInDegree(0, h.accept, h) ||
-           hasGreaterInDegree(1, h.acceptEod, h));
+    assert(in_degree(h.accept, h) || in_degree(h.acceptEod, h) > 1);
 
     unique_ptr<RoseInGraph> igp = buildRose(h, false, cc);
     if (igp && rose.addRose(*igp, prefilter)) {
@@ -2924,6 +2932,7 @@ bool finalChanceRose(RoseBuild &rose, const NGHolder &h, bool prefilter,
         add_edge(v, a, RoseInEdgeProps(rhs, 0U), ig);
     }
 
+    renumber_vertices(ig);
     calcVertexOffsets(ig);
 
     return rose.addRose(ig, prefilter, true /* final chance */);
@@ -2936,8 +2945,7 @@ bool checkRose(const ReportManager &rm, const NGHolder &h, bool prefilter,
     }
 
     // We should have at least one edge into accept or acceptEod!
-    assert(hasGreaterInDegree(0, h.accept, h) ||
-           hasGreaterInDegree(1, h.acceptEod, h));
+    assert(in_degree(h.accept, h) || in_degree(h.acceptEod, h) > 1);
 
     unique_ptr<RoseInGraph> igp;
 
