@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015-2016, Intel Corporation
+ * Copyright (c) 2015-2017, Intel Corporation
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -72,7 +72,7 @@ void getForwardReach(const NGHolder &g, u32 top, map<s32, CharReach> &look) {
         if (v == g.startDs) {
             continue;
         }
-        if (g[e].top == top) {
+        if (contains(g[e].tops, top)) {
             curr.insert(v);
         }
     }
@@ -261,7 +261,7 @@ void findForwardReach(const RoseGraph &g, const RoseVertex v,
     for (const auto &e : out_edges_range(v, g)) {
         RoseVertex t = target(e, g);
         if (!g[t].left) {
-            DEBUG_PRINTF("successor %zu has no leftfix\n", g[t].idx);
+            DEBUG_PRINTF("successor %zu has no leftfix\n", g[t].index);
             return;
         }
         rose_look.push_back(map<s32, CharReach>());
@@ -460,17 +460,41 @@ void findFloodReach(const RoseBuildImpl &tbi, const RoseVertex v,
     }
 }
 
+static
+map<s32, CharReach> findLiteralReach(const rose_literal_id &lit) {
+    map<s32, CharReach> look;
+
+    u32 i = lit.delay + 1;
+    for (auto it = lit.s.rbegin(), ite = lit.s.rend(); it != ite; ++it) {
+        look[0 - i] |= *it;
+        i++;
+    }
+
+    return look;
+}
+
 static
 map<s32, CharReach> findLiteralReach(const RoseBuildImpl &build,
                                      const RoseVertex v) {
+    bool first = true;
     map<s32, CharReach> look;
     for (u32 lit_id : build.g[v].literals) {
         const rose_literal_id &lit = build.literals.right.at(lit_id);
+        auto lit_look = findLiteralReach(lit);
 
-        u32 i = lit.delay + 1;
-        for (auto it = lit.s.rbegin(), ite = lit.s.rend(); it != ite; ++it) {
-            look[0 - i] |= *it;
-            i++;
+        if (first) {
+            look = move(lit_look);
+            first = false;
+        } else {
+            for (auto it = look.begin(); it != look.end();) {
+                auto l_it = lit_look.find(it->first);
+                if (l_it == lit_look.end()) {
+                    it = look.erase(it);
+                } else {
+                    it->second |= l_it->second;
+                    ++it;
+                }
+            }
         }
     }
 
@@ -585,7 +609,7 @@ bool getTransientPrefixReach(const NGHolder &g, u32 lag,
     NFAVertex v = *(inv_adjacent_vertices(g.accept, g).first);
     u32 i = lag + 1;
     while (v != g.startDs) {
-        DEBUG_PRINTF("i=%u, v=%u\n", i, g[v].index);
+        DEBUG_PRINTF("i=%u, v=%zu\n", i, g[v].index);
         if (is_special(v, g)) {
             DEBUG_PRINTF("special\n");
             return false;
